package smsqesprites;
/**
 * A file filter for image files.
 * 
 * @author and copyright (c) Wolfgang Lenerz 2013
 */
public class ImageFilter extends javax.swing.filechooser.FileFilter 
{
    private static final String EXTENSIONS=".bmp.gif.tif.tiff.jpg.jpeg.png";
    
    /**
     * Accepts directories, bmp,tif(f),jp(e)g,png and gif files.
     * 
     * @param f the file to check
     * @return true if file is accepted, else false.
     */
    public boolean accept(java.io.File f) 
    {
        if (f.isDirectory()) 
        {
            return true;
        }
        String extension = getExtension(f);
        if (extension != null && EXTENSIONS.contains("."+extension)) 
        {
            return true;
        } 
        else 
        {
            return false;
        }
    }

    /**
     * Gets the description of this filter.
     * 
     * @return the descrition string.
     */
    public String getDescription() 
    {
        return "Image files (jp(e)g, bmp, gif, tif(f), png)";
    }
    
    /**
     * Gets the extension of a file.
     * This presumes that the extension separator is a ".".
     * 
     * @return the extension in lower case or null if none found.
     */
    public static String getExtension(java.io.File f) 
    {
        String s = f.getName();
        int i = s.lastIndexOf('.');
        if (i > 0 &&  i < s.length() - 1) 
        {
            return s.substring(i+1).toLowerCase();
        }
        return null;
    }
    
}