package smsqesprites;

/**
 * The main window of the program.
 * 
 * @author and copyright (c) Wolfgang Lenerz 2013
 */
public class MainFrame extends javax.swing.JFrame
{
    
    private SpritePanel spritePanel;
    private javax.swing.JFileChooser fc;
    private boolean doReflection=false;
    private ImageFilter imfi;
    private boolean doPreview=false;
    
   /**
     * Creates new form MainFrame.
     */
    public MainFrame() 
    {
        initComponents();
        this.modeSpinner.setValue("64");
        this.gapSpinner.setValue(2);
        this.setIconImage(getFDImage("/images/WolfSoft.gif"));
        java.util.Hashtable<Integer, javax.swing.JLabel> labels =  new java.util.Hashtable<>();
        labels.put(0, new javax.swing.JLabel("-"));
        labels.put(10, new javax.swing.JLabel("+"));
        this.opacitySlider.setLabelTable(labels);
        this.opacitySlider.setPaintLabels(true);
        this.heightSlider.setLabelTable(labels);
        this.heightSlider.setPaintLabels(true);
        this.spritePanel=new SpritePanel();
        addPanel(this.spritePanel);
        this.setMinimumSize(new java.awt.Dimension(this.getWidth(),this.getHeight()));
        this.fc=new javax.swing.JFileChooser("/home/wolf/smsqe");
        this.imfi=new ImageFilter();
        this.opacitySlider.addChangeListener(new OpacitySliderListener(this.spritePanel));
        this.heightSlider.addChangeListener(new HeightSliderListener(this.spritePanel));
        this.gapSpinner.addChangeListener(new GapSpinnerListener(this.spritePanel));
   /*     boolean p=true;
            this.previewButton.setEnabled(p);
            this.addReflectionButton.setEnabled(p);
            this.saveImageButton.setEnabled(p);
            */
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        loadImageButton = new javax.swing.JButton();
        addReflectionButton = new javax.swing.JButton();
        previewButton = new javax.swing.JButton();
        saveImageButton = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        gapSpinner = new javax.swing.JSpinner();
        jSeparator2 = new javax.swing.JSeparator();
        jLabel3 = new javax.swing.JLabel();
        heightSlider = new javax.swing.JSlider();
        jSeparator1 = new javax.swing.JSeparator();
        jLabel2 = new javax.swing.JLabel();
        opacitySlider = new javax.swing.JSlider();
        jSeparator3 = new javax.swing.JSeparator();
        jLabel4 = new javax.swing.JLabel();
        modeSpinner = new javax.swing.JSpinner();
        jPanel3 = new javax.swing.JPanel();
        jMenuBar1 = new javax.swing.JMenuBar();
        optionsMenu = new javax.swing.JMenu();
        fullMaskCheckBox = new javax.swing.JCheckBoxMenuItem();
        uncachbleSpriteCheckBox = new javax.swing.JCheckBoxMenuItem();
        jMenu1 = new javax.swing.JMenu();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("SMSQESprite");
        setIconImages(null);

        jPanel1.setLayout(new java.awt.GridLayout(1, 0));

        loadImageButton.setText("Load an Image");
        loadImageButton.setToolTipText("Load an image to convert into an SMSQ/E sprite");
        loadImageButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadImageButtonActionPerformed(evt);
            }
        });
        jPanel1.add(loadImageButton);

        addReflectionButton.setText("Add reflection");
        addReflectionButton.setToolTipText("Add a reflection to an image");
        addReflectionButton.setEnabled(false);
        addReflectionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addReflectionButtonActionPerformed(evt);
            }
        });
        jPanel1.add(addReflectionButton);

        previewButton.setText("Show preview");
        previewButton.setToolTipText("Show a preview of what the sprite might look loke in the selected Mode.");
        previewButton.setEnabled(false);
        previewButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                previewButtonActionPerformed(evt);
            }
        });
        jPanel1.add(previewButton);

        saveImageButton.setText("Save as SMSQ/E sprite");
        saveImageButton.setToolTipText("Save the image as an SMSQE sprite, this will append the extension \".spr\" to the name");
        saveImageButton.setActionCommand("Save as SMSQ/E sprite in the currently selected mode");
        saveImageButton.setEnabled(false);
        saveImageButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveImageButtonActionPerformed(evt);
            }
        });
        jPanel1.add(saveImageButton);

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Reflection parameters"));
        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.LINE_AXIS));

        jLabel1.setText("Gap:");
        jPanel2.add(jLabel1);

        gapSpinner.setModel(new javax.swing.SpinnerNumberModel(10, 0, 40, 1));
        gapSpinner.setToolTipText("The gap between the image and its reflection, in pixels");
        jPanel2.add(gapSpinner);

        jSeparator2.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jPanel2.add(jSeparator2);

        jLabel3.setText(" Height: ");
        jPanel2.add(jLabel3);

        heightSlider.setMajorTickSpacing(2);
        heightSlider.setMaximum(10);
        heightSlider.setMinorTickSpacing(1);
        heightSlider.setPaintTicks(true);
        heightSlider.setToolTipText("Height of the reflection");
        heightSlider.setValue(5);
        jPanel2.add(heightSlider);

        jSeparator1.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jPanel2.add(jSeparator1);

        jLabel2.setText(" Opacity: ");
        jPanel2.add(jLabel2);

        opacitySlider.setMajorTickSpacing(2);
        opacitySlider.setMaximum(10);
        opacitySlider.setMinorTickSpacing(1);
        opacitySlider.setPaintTicks(true);
        opacitySlider.setSnapToTicks(true);
        opacitySlider.setToolTipText("Opacity of the first line of the reflection");
        opacitySlider.setValue(5);
        jPanel2.add(opacitySlider);

        jSeparator3.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jPanel2.add(jSeparator3);

        jLabel4.setText("Mode : ");
        jPanel2.add(jLabel4);

        modeSpinner.setModel(new javax.swing.SpinnerListModel(new String[] {"4", "16", "32", "64"}));
        modeSpinner.setToolTipText("Mode of the SMSQE sprite");
        jPanel2.add(modeSpinner);

        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.LINE_AXIS));

        optionsMenu.setText("Options");

        fullMaskCheckBox.setText("Always full mask");
        fullMaskCheckBox.setToolTipText("If you check this, the mask of the sprite will always be full (no alpha values)");
        optionsMenu.add(fullMaskCheckBox);

        uncachbleSpriteCheckBox.setText("Uncachable sprite");
        uncachbleSpriteCheckBox.setToolTipText("If you select this, the sprite will never be cached by SMSQE. Useful (only) for debugging a sprite.");
        optionsMenu.add(uncachbleSpriteCheckBox);

        jMenuBar1.add(optionsMenu);

        jMenu1.setText("?");
        jMenu1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                jMenu1MousePressed(evt);
            }
        });
        jMenu1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenu1ActionPerformed(evt);
            }
        });
        jMenuBar1.add(jMenu1);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel3, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(5, 5, 5)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(5, 5, 5)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, 229, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Save the image as an SMSQE sprite.
     * 
     * @param evt ignored.
     */
    private void saveImageButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveImageButtonActionPerformed
        this.fc.setFileFilter(null);
        if (this.fc.showSaveDialog(this)==javax.swing.JFileChooser.APPROVE_OPTION)
        {
            java.io.File f= this.fc.getSelectedFile();      // file to save to.
            if (f==null)
                return;                                     // none
            String ext=ImageFilter.getExtension(f);               
            if ( ext==null || (!ext.equals("spr")))
            {
                f=new java.io.File (f.getAbsolutePath()+".spr");
            }
            String p=(String)this.modeSpinner.getValue();
            int mode=64;                                    // default value for sprite mode
            try
            {
                mode=Integer.parseInt(p);
            }
            catch(Exception e)
            {
                /*nop*/                                     // huh? use default value
            }
            this.spritePanel.makeSprite(f,mode,this.uncachbleSpriteCheckBox.isSelected(),this.fullMaskCheckBox.isSelected());
        }
    }//GEN-LAST:event_saveImageButtonActionPerformed

    /**
     * Shows a filechooser to let you choose an image, and loads it.
     * 
     * @param evt ignored
     */
    private void loadImageButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadImageButtonActionPerformed
        this.fc.setFileFilter(this.imfi);
        if (this.fc.showOpenDialog(this)==javax.swing.JFileChooser.APPROVE_OPTION)
        {
            java.io.File f= this.fc.getSelectedFile();
            boolean p;
            if (this.spritePanel.setImage(f))
            {
                pack();
                p=true;
            }
            else
            {
                p=false;
            }
            this.previewButton.setText("Show preview");
            this.addReflectionButton.setText("Add reflection");
            this.doPreview=false;
            this.doReflection=false;
            this.previewButton.setEnabled(p);
            this.addReflectionButton.setEnabled(p);
            this.saveImageButton.setEnabled(p);
        }
    }//GEN-LAST:event_loadImageButtonActionPerformed

    /**
     * Causesa  reflection to be added/removed to/from the image.
     * 
     * @param evt ignored
     */
    private void addReflectionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addReflectionButtonActionPerformed
        this.doReflection=!this.doReflection;
        if (this.doReflection)
        {
            this.addReflectionButton.setText("No reflection");
        }
        else
        {
            this.addReflectionButton.setText("Add reflection");
        }
        this.spritePanel.showReflection(this.doReflection);
        pack();
    }//GEN-LAST:event_addReflectionButtonActionPerformed

    
    private void previewButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_previewButtonActionPerformed
        this.doPreview=!this.doPreview;
        String p=(String)this.modeSpinner.getValue();
        int mode=64;                                    // default value for sprite mode
        try
        {
            mode=Integer.parseInt(p);
        }
        catch(Exception e)
        {
            /*nop*/                                     // huh? use default value
        }
        if (mode==64)
        {
            javax.swing.JOptionPane.showMessageDialog(rootPane, "<html>There is no preview for this mode, the sprite will be the same as the image <p>if viewed in an SMSQE that can truely display 24 bit colours (none such exists as yet).</html>", "No preview available", javax.swing.JOptionPane.ERROR_MESSAGE);
            return;
        }
        this.doPreview=this.spritePanel.setPreview(this.doPreview,mode);
        this.previewButton.setText(this.doPreview?"Hide preview":"Show preview");
    }//GEN-LAST:event_previewButtonActionPerformed

    private void jMenu1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenu1ActionPerformed
        
        java.net.URL imgURL = MainFrame.class.getResource("/images/WolfSoft2.gif");// get my icon
        if (imgURL!=null)
        {
            javax.swing.ImageIcon image = new javax.swing.ImageIcon(imgURL);            // and make it into correct icon
            javax.swing.JOptionPane.showMessageDialog(this,"SMSQESprites \nCopyright (C) Wolfgang Lenerz 2013\n v.1.02","SMSQESprites ",1,image);
        }
        else
        {
            javax.swing.JOptionPane.showMessageDialog(this,"SMSQESprites \nCopyright (C) Wolfgang Lenerz 2013\n v.1.02","SMSQESprites ",1);
        }
        
    }//GEN-LAST:event_jMenu1ActionPerformed

    private void jMenu1MousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jMenu1MousePressed
        jMenu1ActionPerformed(null);
    }//GEN-LAST:event_jMenu1MousePressed

    /**
     * Main method. Set LAF, create the object.
     * 
     * @param args the command line arguments - ignored.
     */
    public static void main(String args[]) 
    {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try 
        {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) 
            {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } 
        catch (ClassNotFoundException ex) 
        {
            java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } 
        catch (InstantiationException ex) 
        {
            java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } 
        catch (IllegalAccessException ex) 
        {
            java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        catch (javax.swing.UnsupportedLookAndFeelException ex) 
        {
            java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new MainFrame().setVisible(true);
            }
        });
    }
    
    /**
     * Adds the SpritePanel to the form.
     * @param spritePanel 
     */
     private void addPanel(SpritePanel spritePanel)
     {
      //  this.jPanel3.setLayout(new javax.swing.BoxLayout(getContentPane(), WIDTH));
        this.jPanel3.add(spritePanel);
        pack();
     }
     
    /**
     * Get the image for the title & minimized bars.
     * @param filename of the image to load.
     * @return the image or null.
     */
    public static java.awt.Image getFDImage(String filename)
    {
        java.net.URL imgURL = MainFrame.class.getResource(filename);
        if (imgURL != null)
        {
            return new javax.swing.ImageIcon(imgURL).getImage();
        }
        else
        {
            return null;
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addReflectionButton;
    private javax.swing.JCheckBoxMenuItem fullMaskCheckBox;
    private javax.swing.JSpinner gapSpinner;
    private javax.swing.JSlider heightSlider;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JButton loadImageButton;
    private javax.swing.JSpinner modeSpinner;
    private javax.swing.JSlider opacitySlider;
    private javax.swing.JMenu optionsMenu;
    private javax.swing.JButton previewButton;
    private javax.swing.JButton saveImageButton;
    private javax.swing.JCheckBoxMenuItem uncachbleSpriteCheckBox;
    // End of variables declaration//GEN-END:variables
}
